
/*
 * we use jquery's onReady() functionality to wait until the page loads before making
 * our REST calls.
 * 
 */
$(function() {

	/*
	 * we hard-code our city ID here to something appropriate.  4932869 is cityid 
	 * for Chelmsford, MA (where PUGChallenge 2015 is being held).
	 * 
	 * 
	 */
	
	var cityid = "4932869";

	var currentUrl = "/cgi-bin/cgiip.exe/rest/weather/" + cityid;
	var forecastUrl = "/cgi-bin/cgiip.exe/rest/forecast/" + cityid;

	/*
	 * current data and forecast data are served up separately.  We could merge them,
	 * but openweathermap.org splits them, so for fun we split them in our API as well.
	 * 
	 */
	$.ajax({
		url : currentUrl,
		datatype : 'json',
		success : function(data) {
			$("#temperature").html(data.main.temp);
			$("#pressure").html(data.main.pressure);
			$("#humidity").html(data.main.humidity);
			$("#city").html(data.name);
		}
	});

	/*
	 * 
	 * The grid and the chart can share a data source for forecast.  Or they can use separate ones,
	 * but having them separate results in two calls to openweathermap.org and we get back the same data
	 * so we just share the datasource between the controls.
	 * 
	 * Since we reformatted the data and used a temp-table, the "data" property is "ttforecast".  We could
	 * change this to something like "forecasts" by using SERIALIZE-NAME property
	 */
	var dataSource = new kendo.data.DataSource({
		type : 'json',
		transport : {
			read : forecastUrl
		},
		schema : {
			data : 'ttforecast',
			model : {
				fields : {
					temp : {
						type : 'number'
					},
					temp_min : {
						type : 'number'
					},
					temp_max : {
						type : 'number'
					},
					humidity : {
						type : 'number'
					},
					pressure : {
						type : 'number'
					},
					rain : {
						type : 'number'
					},
					dt : {
						type : 'date',
						parse : function(val) {
							return new Date(val);
						}
					}
				}
			}
		}

	});

	/*
	 * the grid is really simple, we just pick out the columns we want from the data source
	 * and give them titles.  Notice we show different data in the grid and the chart
	 */
	$("#forecast").kendoGrid({
		dataSource : dataSource,
		autoBind: false,
		columns : [ {
			title : "Humidity",
			field : 'humidity'
		}, {
			title : "Pressure",
			field : "pressure"
		}, {
			title : "Temp Min",
			field : "temp_min"
		}, {
			title : "Temp Max",
			field : "temp_max"
		} ]
	});

	/*
	 * The chart is a bit more complex, because we want two value axis with separate formatting and colors.
	 * We also set the rain and temperature series to use separate axis, so they scale differently within 
	 * the same overlapping space.  The time axis step is modified since we have a lot of data points and we
	 * don't want the time values to overlap.
	 */
	$("#chart").kendoChart({
		dataSource : dataSource,
		autoBind: false,
		seriesDefaults : {
			tooltips : true
		},
		legend : {
			position : 'top'
		},
		title : "Forecast",
		series : [ {
			name : "Precipitation",
			type : "column",
			field : 'rain',
			axis : 'rain',
			color : '#24A382',
		}, {
			name : "Temperature",
			type : "line",
			style : 'smooth',
			field : 'temp',
			axis : 'temp',
			color : '#FF4E00',
		} ],
		categoryAxis : {
			field : 'dt',
			minorGridLines : {
				visible : false
			},
			baseUnitStep : 5,
			majorGridLines : {
				visible : false
			}

		},
		valueAxes : [ {
			name : 'rain',
			color : '#24A382',
			min : 0,
			max : 3,
			labels : {
				format : "{0}mm"
			}
		}, {
			name : 'temp',
			color : '#FF4E00',
			max : 110,
			labels : {
				format : "{0}&deg;"
			}

		} ]

	});
	
	/**
	 * we use 'autoBind' = false so the grid and chart don't try to both load the same data.  Instead
	 * we load the data source down here after the chart and grid have been defined.
	 */
	dataSource.read();

});