/*
This program retrieves the forecast for a particular city.  This requires
cityid in order to fetch the forecast from http://openweathermap.org.

*/


using OpenEdge.Net.HTTP.*.
using Progress.Json.ObjectModel.JsonObject from propath.
using Progress.Json.ObjectModel.JsonArray from propath.

define input parameter cityid as character no-undo.
define input parameter webstream as handle no-undo.

define variable apikey    as character     no-undo.
define variable url       as character     no-undo init "http://api.openweathermap.org/data/2.5/forecast".
define variable epoch     as datetime      no-undo initial 1/1/1970.

define variable response  as longchar      no-undo.

define variable oClient   as IHttpClient.
define variable oRequest  as IHttpRequest.
define variable oResponse as IHttpResponse.

define temp-table ttforecast no-undo
    field temp       as decimal
    field temp_min   as decimal
    field temp_max   as decimal
    field rain       as decimal
    field pressure   as decimal
    field sea_level  as decimal
    field grnd_level as decimal
    field humidity   as decimal
    field temp_kf    as decimal
    field dt         as datetime
    .

run weather/getapikey.p (output apikey).

oClient = ClientBuilder:Build():Client.
 
/*
the openweathermap.org API doesn't use "REST" compliant URIs.  Instead the city id is 
passed as part of the query.  So we have to format the URL the way it wants or we get a 
HTTP 500 back.  

*/
url = url + "?id=" + cityid + "&units=imperial". 
 
oRequest = RequestBuilder:Get(url):AddHeader("x-api-key", apikey):Request.

oResponse = oClient:Execute(oRequest). 

/*
request data comes back in a non-kendo UI friendly format.  Here we need to strip out the
parts we don't care about, and reformat some of it into a nice array of objects containing
only the data we care about.  

*/

define variable forecastJson as JsonObject no-undo. 
forecastJson = cast(oResponse:Entity, JsonObject).
    
define variable forecastList as JsonArray no-undo.
forecastList = forecastJson:GetJsonArray("list").

define variable i as integer no-undo.
do i = 1 to forecastList:Length:
    define variable main     as JsonObject no-undo.
    define variable rain as JsonObject no-undo. 
    define variable forecast as JsonObject no-undo.
    forecast = forecastList:GetJsonObject(i).
    main = forecast:GetJsonObject("main").
    
    /* 
        openweathermap.org leaves out "rain" node if there isn't any...of course this isn't documented :)
    
    */
    if forecast:Has("rain") then do:
        rain = forecast:GetJsonObject("rain").
    end.
   
    create ttforecast.
    ttforecast.temp = main:GetDecimal("temp").
    ttforecast.temp_min = main:GetDecimal("temp_min").
    ttforecast.temp_max = main:GetDecimal("temp_max").
    ttforecast.pressure = main:GetDecimal("pressure").
    ttforecast.sea_level = main:GetDecimal("sea_level").
    ttforecast.grnd_level = main:GetDecimal("grnd_level").
    ttforecast.humidity = main:GetDecimal("humidity").
    
    /*
      might not be any rain today so this might be missing
    */
    if rain <> ? then do:
        ttforecast.rain = rain:GetDecimal("3h").
    end.
    ttforecast.dt = add-interval(epoch, forecast:GetInteger("dt"), "seconds").
end.

/*
Formatting the data amounts of converting it into a temp-table and writing the temp-table
out to the webstream.
*/
temp-table ttforecast:write-json("stream-handle", webstream:handle ).
